import commands

default_parameters = {}


def assert_verify_message(message):
    """
    Verifies that the message is not None,
    'payload' exists and 'payload' is not None.
    :param message:
    :return:
    """
    assert message is not None
    assert 'payload' in message
    assert message['payload'] is not None


def assert_entity_is_registered(cli, entity_id):
    """
    Asserts that the entity is registered.
    :param cli:
    :param entity_id:
    :return:
    """
    result = commands.query_entity_is_registered(cli, entity_id)
    assert_verify_message(result)
    assert result['payload']['result'] is True
    return result


def assert_await_entity_registered(cli, entity_id, is_registered=True, timeout_seconds=60):
    """
    Asserts that we successfully awaited for the registration state of the entity. If the timeout passes
    or the expression is_registered != actual state, then it will fail.
    :param cli:
    :param entity_id:
    :param is_registered: (True | False) the state change we are waiting for.
    :param timeout_seconds: The amount of time to wait for a change before fail.
    :return:
    """
    result = commands.await_entity_registered(cli, entity_id, is_registered, timeout_seconds)
    assert_verify_message(result)
    assert result['payload']['success'] is True
    return result


def assert_invoke_entity_method(cli, entity_id, method_name, parameters=default_parameters):
    """
    Asserts that the invocation has not errored.
    :param cli:
    :param entity_id:
    :param method_name: The public method name we are invoking
    :param parameters: A {} of values to pass to the method. Converted to IDynamicObject.
    :return:
    """
    result = commands.invoke_entity_method(cli, entity_id, method_name, parameters)
    assert_verify_message(result)
    assert result['payload']['is_error'] is False, result['payload']['error_message']
    return result


def assert_fetch_entity(cli, entity_id):
    """
    Asserts that the returned test_entity is not None.
    :param cli:
    :param entity_id:
    :return:
    """
    result = commands.fetch_entity(cli, entity_id)
    assert_verify_message(result)
    assert result['payload']['test_entity'] is not None
    return result



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import logging
import time
import socket
import struct
import json
import message


class Client:
    MESSAGE_LENGTH_SIZE = 4
    BUFFER_SIZE = 1024

    DEFAULT_TCP_IP = '127.0.0.1'
    DEFAULT_TCP_PORT = 13000
    DEFAULT_TCP_FORWARD_PORT = 13001
    DEFAULT_NUM_CONNECTION_RETRIES = 12
    DEFAULT_CONNECTION_RETRY_TIMEOUT = 5

    def __init__(self, tcp_ip=DEFAULT_TCP_IP, tcp_port=DEFAULT_TCP_PORT):
        if tcp_ip is None:
            raise ValueError("tcp_ip cannot be None!")

        if tcp_port is None:
            raise ValueError("tcp_port cannot be None!")

        self.tcp_ip = tcp_ip
        self.tcp_port = tcp_port

    def connect(self, num_retries=DEFAULT_NUM_CONNECTION_RETRIES, timeout=DEFAULT_CONNECTION_RETRY_TIMEOUT):
        while num_retries > 0:
            try:
                logging.info("Connecting with ip: {0} and port: {1}".format(self.tcp_ip, self.tcp_port))
                self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                self.socket.connect((self.tcp_ip, self.tcp_port))
                break
            except Exception as e:
                logging.error(e)
                num_retries -= 1
                time.sleep(timeout)

        if num_retries <= 0:
            self.stop()
            raise Exception("Could not connect to server at {0}:{1}".format(self.tcp_ip, self.tcp_port))

    def stop(self):
        self.socket.shutdown(socket.SHUT_WR)
        self.socket.close()

    def read_message(self):
        """
        A blocking call that returns a full IMessage from the server.
        :return: Message
        """

        # Read content length...
        content_length_binary = self.socket.recv(self.MESSAGE_LENGTH_SIZE)

        while len(content_length_binary) < self.MESSAGE_LENGTH_SIZE:
            content_length_binary += self.socket.recv(self.MESSAGE_LENGTH_SIZE - len(content_length_binary))

        content_length = struct.unpack('>HH', content_length_binary)[1]

        # Read content in full...
        content_binary = self.socket.recv(self.BUFFER_SIZE)

        while len(content_binary) < content_length:
            content_binary += self.socket.recv(self.BUFFER_SIZE)

        msg = json.loads(content_binary)
        logging.info("Receive: {0}".format(msg))

        return msg

    def send_message(self, msg):
        """
        Serializes the message and sends to server.
        :param msg:
        :return: None
        """
        if msg is None:
            raise ValueError('message cannot be None!')

        if not isinstance(msg, message.Message):
            raise ValueError('message must be a type of Message')

        message_json = json.dumps(msg.__dict__)
        message_length = len(message_json)
        message_length_binary = struct.pack('>I', message_length)

        logging.info("Send: {0}".format(message_json))

        self.socket.send(message_length_binary)
        self.socket.send(message_json)



#!/usr/bin/env python
# -*- coding: utf-8 -*-


class Message:
    """
    Represents a message that is serialized to json as {'type':..., 'payload':...}
    """

    type = None
    payload = None

    def __init__(self, message_type, payload):
        if message_type is None:
            raise ValueError('message_type cannot be None!')

        if payload is None:
            raise ValueError('payload cannot be None!')

        if not isinstance(payload, dict):
            raise ValueError('payload must be a type of dict')

        self.type = message_type
        self.payload = payload



import message

DEFAULT_TIMEOUT_SECONDS = 60
DEFAULT_ENTITY_REGISTERED = True


def query_entity_is_registered(cli, entity_id):
    """
    Requests status on whether an entity is registered or not.
    :param cli:
    :param entity_id:
    :return:
    """

    message_payload = {
        "entity_id": entity_id
    }
    msg = message.Message("query.entity.isRegistered", message_payload)
    cli.send_message(msg)
    return cli.read_message()


def await_entity_registered(cli, entity_id, is_registered=DEFAULT_ENTITY_REGISTERED, timeout_seconds=DEFAULT_TIMEOUT_SECONDS):
    """
    Waits for an entity to become registered, then continues.
    :param cli:
    :param entity_id:
    :param is_registered: Whether or not to await for registered state (True | False)
    :param timeout_seconds: How long until this returns with failure
    :return:
    """
    message_payload = {
        "entity_id": entity_id,
        "is_registered": is_registered,
        "timeout": timeout_seconds
    }
    msg = message.Message("await.entity.registered", message_payload)
    cli.send_message(msg)
    return cli.read_message()


def fetch_entity(cli, entity_id):
    """
    Fetches a serialized representation of the entity. If it is
    ITestEntity<T>, then the model is serialized as well.
    :param cli:
    :param entity_id:
    :return:
    """
    message_payload = {
        "entity_id": entity_id
    }
    msg = message.Message("fetch.entity", message_payload)
    cli.send_message(msg)
    return cli.read_message()


def invoke_entity_method(cli, entity_id, method_name, parameters):
    """
    Invokes a method on the ITestEntity. It then returns the serialized
    value.
    :param cli:
    :param entity_id:
    :param method_name:
    :param parameters:
    :return:
    """
    message_payload = {
        "entity_id": entity_id,
        "method_name": method_name,
        "parameters": parameters
    }
    msg = message.Message("invoke.entity.method", message_payload)
    cli.send_message(msg)
    return cli.read_message()